package se.iqpd.checkout.plus;

import android.annotation.SuppressLint;
import android.app.ActivityManager;
import android.app.ActivityManager.RunningServiceInfo;
import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.ServiceConnection;
import android.graphics.Bitmap;
import android.os.IBinder;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.ActivityObject;
import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.DependsOn;
import anywheresoftware.b4a.BA.Events;
import anywheresoftware.b4a.BA.Hide;
import anywheresoftware.b4a.BA.Permissions;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;

import com.ingenico.pclservice.IPclServiceCallback;
import com.ingenico.pclservice.PclService;
import com.ingenico.pclservice.PclService.LocalBinder;

@ActivityObject
@Author("IQP Design AB")
@Events(values={
        "onCompanionStateChange(NewState As String)",
        "onCompanionBarcodeRead(Barcode As String)",
        "onStatusUpdate(EventName As String, Status As String)",
        "onResult(EventName As String, Result As String)"})
@ShortName("ISMPCompanion") // this is the name as it appear on the IDE
@Permissions(values = {"android.permission.ACCESS_NETWORK_STATE",
        "android.permission.BLUETOOTH",
        "android.permission.BLUETOOTH_ADMIN",
        "android.permission.ACCESS_WIFI_STATE",
        "android.permission.READ_PHONE_STATE"
})
@Version(0.80f)
@DependsOn(values = {"PclServiceLib","android-support-v4"})
public class ISMP_Companion
{

	private BA Basic4Android = null;
	private String _EventName;

	@Hide
	protected PclService mPclService = null;
	
	private PclServiceConnection mServiceConnection;
	private StateReceiver m_StateReceiver = null;
	
	private BarCodeReceiver m_BarCodeReceiver = null;
	private boolean m_BarCodeActivated = false;

	class PclServiceConnection implements ServiceConnection
	{
		public void onServiceConnected(ComponentName className, IBinder boundService )
		{
			LocalBinder binder = (LocalBinder) boundService;
			mPclService = (PclService) binder.getService();

			BA.Log("ISMPWrapper (onServiceConnected): " + className);
			initStateReceiver();
			mPclService.registerCallback(mCallback);
			
    		Basic4Android.raiseEvent(Basic4Android.activity, _EventName + "_connectionstate","CONNECTED");
		}
		
		public void onServiceDisconnected(ComponentName className)
		{
			BA.Log("ISMPWrapper (onServiceDisconnected): " + className);
			mPclService = null;
			Basic4Android.raiseEvent(Basic4Android.activity, _EventName + "_connectionstate","DISCONNECTED");
			releaseStateReceiver();
		}
	};

	//***************************************************
	//************** PCLService *************************
	//***************************************************
	
	private void initService()
	{
		mServiceConnection = new PclServiceConnection();
		Intent intent = new Intent(BA.applicationContext, PclService.class);
		BA.applicationContext.bindService(intent, mServiceConnection, Context.BIND_AUTO_CREATE);
		BA.Log("ISMPWrapper (initService)");
	}
	
	// You can call this method in onDestroy for instance
	
	private void releaseService()
	{
		BA.applicationContext.unbindService(mServiceConnection);
		BA.Log("ISMPWrapper (releaseService)");
	}
	
	private boolean isPCLServiceRunning()
	{
	    ActivityManager manager = (ActivityManager) BA.applicationContext.getSystemService(Context.ACTIVITY_SERVICE);
	    
	    for (RunningServiceInfo service : manager.getRunningServices(Integer.MAX_VALUE))
	    {
	        if (PclService.class.getName().equals(service.service.getClassName()))
	        {
	        	BA.Log("ISMPWrapper(CommonActivity)(isPCLServiceRunning) = true");
	            return true;
	        }
	    }
	    BA.Log("ISMPWrapper(CommonActivity)(isPCLServiceRunning) = false");
	    return false;
	}
	
	//**************************************************************
	//***** Basic 4 Android ****************************************
	//**************************************************************
	
	/*
	 * Initialize the Companion, Needs to be called in Activity_Create
	 */
	public void Initialize(final BA ba, String EventName)
	{
		
		Basic4Android = ba;
		_EventName = EventName;
		
		BA.Log("ISMPWrapper: Initialize");

		System.loadLibrary("pclapi");
		System.loadLibrary("pclnetwork");
		System.loadLibrary("pcltools");
		System.loadLibrary("tlvtree");

		initService();
		
	}
	
	/*
	 * Should be called when application pauses
	 */
	public void Pause()
	{
		BA.Log("ISMPWrapper: Pause");
		releaseBarCodeReceiver();
		releaseStateReceiver();
		closeBarCode();
	}
	/*
	 * Should be called when application resumes
	 */
	public void Resume()
	{
		BA.Log("ISMPWrapper: Resume");
        initService();
		initStateReceiver();
		initBarCodeReceiver();
		openBarCode();
	}
	
	/*
	 * Should be called when application ends
	 */
	public void Destroy()
	{
		releaseService();
	}
	
	/*
	 * Check if Companion is Initialized
	 */
    public Boolean IsInitialized()
    {
        if (mPclService == null)
        {
            BA.Log("ISMPWrapper(IsInitialized): IsInitialized=false");
            return false;
        }
        else
        {
            BA.Log("ISMPWrapper(IsInitialized): IsInitialized=true");
            return true;   
        }
    }
    
    public Boolean IsPCLRunning()
    {
        if (isPCLServiceRunning() == false)
        {
            BA.Log("ISMPWrapper(IsInitialized): IsInitialized=false");
            return false;
        }
        else
        {
            BA.Log("ISMPWrapper(IsInitialized): IsInitialized=true");
            return true;   
        }
    }
    
    //***************************************************
    //*********** STATE RECIEVER ON / OFF****************
    //***************************************************
    
    private class StateReceiver extends BroadcastReceiver
    {
    	@SuppressLint("UseValueOf")
    	public void onReceive(Context context, Intent intent)
    	{
    		String state = intent.getStringExtra("state");
    		Basic4Android.raiseEvent(Basic4Android.activity, _EventName + "_connectionstate",state);
    	}
    }
    
    private void initStateReceiver()
    {
    	if(m_StateReceiver == null)
    	{
    		m_StateReceiver = new StateReceiver();
    		IntentFilter intentfilter = new IntentFilter("com.ingenico.pclservice.intent.action.STATE_CHANGED");
    		BA.applicationContext.registerReceiver(m_StateReceiver, intentfilter);
    	}
    }
    // Deinitialize broadcast receiver
    private void releaseStateReceiver()
    {
    	if(m_StateReceiver != null)
    	{
    		BA.applicationContext.unregisterReceiver(m_StateReceiver);
    		m_StateReceiver = null;
    	}
    }

    //***************************************************
    //*********** BARCODE RECIEVER ON / OFF**************
    //***************************************************

	private class BarCodeReceiver extends BroadcastReceiver
	{
		@SuppressLint("UseValueOf")
		public void onReceive(Context context, Intent intent)
		{
			byte abyte0[] = intent.getByteArrayExtra("barcode");
			String BarCodeStr = new String(abyte0);
			Basic4Android.raiseEvent(Basic4Android.activity, _EventName + "_oncompanionbarcoderead",BarCodeStr);
			
		}

	}
	
    private void initBarCodeReceiver()
    {
    	if(m_BarCodeReceiver == null)
    	{
    		m_BarCodeReceiver = new BarCodeReceiver();
	    	IntentFilter intentfilter = new IntentFilter("com.ingenico.pclservice.action.BARCODE_EVENT");
	    	BA.applicationContext.registerReceiver(m_BarCodeReceiver, intentfilter);
    	}
    }
    private void releaseBarCodeReceiver()
    {
    	if(m_BarCodeReceiver != null)
    	{
    		BA.applicationContext.unregisterReceiver(m_BarCodeReceiver);
    		m_BarCodeReceiver = null;
    	}
    }

    public Boolean openBarCode()
	{
    	BA.Log("openBarCode" );
		if((mPclService != null) && !m_BarCodeActivated)
			m_BarCodeActivated = setBarCodeActivation(true);
		
		return m_BarCodeActivated;
	}
	public Boolean closeBarCode()
	{
		BA.Log("closeBarCode" );
		if((mPclService != null) && m_BarCodeActivated)
			m_BarCodeActivated = !setBarCodeActivation(false);
		
		return m_BarCodeActivated;
	}
	public Boolean reopenBarCode()
	{
		if(m_BarCodeActivated)
			closeBarCode();
		return openBarCode();
	}
	private Boolean setBarCodeActivation(boolean activateBarCode)
	{
		boolean result = false;
		byte array [] = null;
		
		if(mPclService != null)
		{
			array = new byte[1];
			{
				if(activateBarCode)
				{
					result = mPclService.openBarcode(array);
					if (result == true)
					{
						if (array[0] != 0)
							result = false;
					}
				}
				else
				{
					mPclService.closeBarcode(array);
					result = true;
				}
			}
			
		}
		
		return result;
	}

    
    private IPclServiceCallback mCallback = new IPclServiceCallback()
    {
    	@Override
    	public void shouldFeedPaper(int lines)
    	{
    	// Update application TextView for instance
    	}
 
		@Override
		public void shouldAddSignature()
		{
			// TODO Auto-generated method stub
			
		}

		@Override
		public void shouldCutPaper()
		{
			// TODO Auto-generated method stub
			
		}

		@Override
		public void shouldDoSignatureCapture(int arg0, int arg1, int arg2,int arg3, int arg4)
		{
			// TODO Auto-generated method stub
			
		}

		@Override
		public int shouldEndReceipt()
		{
			// TODO Auto-generated method stub
			return 0;
		}

		@Override
		public void shouldPrintImage(Bitmap arg0, byte arg1)
		{
			// TODO Auto-generated method stub
			
		}

		@Override
		public void shouldPrintText(String arg0, byte arg1, byte arg2,
				byte arg3, byte arg4, byte arg5, byte arg6)
		{
			// TODO Auto-generated method stub
			
		}

		@Override
		public int shouldStartReceipt(byte arg0)
		{
			// TODO Auto-generated method stub
			return 0;
		}

		@Override
		public void signatureTimeoutExceeded()
		{
			// TODO Auto-generated method stub
			
		}
    };
}